#include <QStringList>

#include "treeitem.h"

TreeItem::TreeItem(const QList<QVariant> &data, TreeItem *parent)
{
    parentItem = parent;
    itemData = data;
	lmo=NULL;

	branchOffset=0;
	branchCount=0;
	lastSelectedBranchNum=-1;
}

TreeItem::~TreeItem()
{
    qDeleteAll(childItems);
}

void TreeItem::appendChild(TreeItem *item)
{
    childItems.append(item);
	if (item->type == Branch)
	{
		if (branchCount==0)
			branchOffset=childItems.count()-1;
		branchCount++;
	}
}

void TreeItem::removeChild(int row)
{
    if (row>=0)
	{
		delete (childItems.at(row) );
		childItems.removeAt (row);
	} else
		qWarning ("TreeItem::removeChild tried to remove non existing item?!\n");
}

TreeItem *TreeItem::child(int row)
{
    return childItems.value(row);
}

int TreeItem::childCount() const
{
    return childItems.count();
}

int TreeItem::columnCount() const
{
    return itemData.count();
}
int TreeItem::row() const
{
    if (parentItem)
        return parentItem->childItems.indexOf(const_cast<TreeItem*>(this));

    return 0;
}

int TreeItem::column() const
{
    return 0;
}

TreeItem *TreeItem::parent()
{
    return parentItem;
}


QVariant TreeItem::data(int column) const
{
    return itemData.value(column);
}

void TreeItem::setHeading (const QString s)
{
	itemData[0]=s;
}

QString TreeItem::getHeading ()
{
	return itemData[0].toString();
}

void TreeItem::setType(const Type t)
{
	type=t;
	itemData[1]=getTypeName();
}

TreeItem::Type TreeItem::getType()
{
	return type;
}

QString TreeItem::getTypeName()
{
	switch (type)
	{
		case Undefined: return QString ("Undefined");
		case MapCenter: return QString ("MapCenter");
		case Branch: return QString ("Branch");
		default: return QString ("TreeItem::getTypeName no typename defined?!");
	}
}

TreeItem* TreeItem::getChildNum(const int &n)
{
	if (n>=0 && n<childItems.count() )
		return childItems.at(n);
	else
		return NULL;
}

TreeItem* TreeItem::getFirstBranch()
{
	if (branchCount>0)
		return getBranchNum (branchOffset);
	else
		return NULL;
}

TreeItem* TreeItem::getLastBranch()
{
	if (branchCount>0)
		return getBranchNum (branchOffset + branchCount-1);
	else
		return NULL;
}


TreeItem* TreeItem::getBranchNum(const int &n)
{
	if (branchCount>0)
		return getChildNum (branchOffset + n);
	else
		return NULL;
}

void TreeItem::setLastSelectedBranch()
{
	if (parentItem)
		parentItem->lastSelectedBranchNum=parentItem->childItems.indexOf(this);
}

TreeItem* TreeItem::getLastSelectedBranch()
{
	return getBranchNum (lastSelectedBranchNum);
}

LinkableMapObj* TreeItem::getLMO()	// FIXME VM should be unnecessary in the end
{
	return lmo;
}

void TreeItem::setLMO(LinkableMapObj *l)
{
	lmo=l;
}

